#include <pt.h>
#include <usbdrv.h>
#include "keycodes.h"
#define REPORT_ID_KEYBOARD 1
// มาโครสำหรับจำลองการหน่วงเวลาใน protothread
#define PT_DELAY(pt,ms,tsVar) \
  tsVar = millis(); \
  PT_WAIT_UNTIL(pt, millis()-tsVar >= (ms));

PROGMEM const char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = 
{
  ///////////////////
  // Keyboard
  ///////////////////
  0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
  0x09, 0x06,                    // USAGE (Keyboard)
  0xa1, 0x01,                    // COLLECTION (Application)
  0x85, 0x01,                    //   REPORT_ID (1)
  0x05, 0x07,                    //   USAGE_PAGE (Keyboard)
  0x19, 0xe0,                    //   USAGE_MINIMUM (Keyboard LeftControl)
  0x29, 0xe7,                    //   USAGE_MAXIMUM (Keyboard Right GUI)
  0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
  0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
  0x75, 0x01,                    //   REPORT_SIZE (1)
  0x95, 0x08,                    //   REPORT_COUNT (8)
  0x81, 0x02,                    //   INPUT (Data,Var,Abs)
  0x95, 0x01,                    //   REPORT_COUNT (1)
  0x75, 0x08,                    //   REPORT_SIZE (8)
  0x25, 0x65,                    //   LOGICAL_MAXIMUM (101)
  0x19, 0x00,                    //   USAGE_MINIMUM (Reserved (no event indicated))
  0x29, 0x65,                    //   USAGE_MAXIMUM (Keyboard Application)
  0x81, 0x00,                    //   INPUT (Data,Ary,Abs)
  0xc0,                          // END_COLLECTION
};

struct ReportKeyboard
{
  /* +----\------+-----+-----+-----+-----+-----+-----+-----+-----+
   * |Byte \ Bit |  7  |  6  |  5  |  4  |  3  |  2  |  1  |  0  |
   * +------\----+-----+-----+-----+-----+-----+-----+-----+-----+
   * |  0        |               Report ID = 1                   |
   * +-----------+-----+-----+-----+-----+-----+-----+-----+-----+
   * |  1        |           Modifiers (shift,ctrl,etc)          |
   * +-----------+-----+-----+-----+-----+-----+-----+-----+-----+
   * |  2        |                 Key Code                      |
   * +-----------+-----+-----+-----+-----+-----+-----+-----+-----+
   */
  uint8_t  report_id;
  uint8_t  modifiers;
  uint8_t  key_code;
};

ReportKeyboard reportKeyboard;

// Protothread states
struct pt up_pt;
struct pt down_pt;
struct pt left_pt;
struct pt right_pt;
struct pt space_pt;

////////////////////////////////////////////////////////////////
// Automatically called by usbpoll() when host makes a request
////////////////////////////////////////////////////////////////
usbMsgLen_t usbFunctionSetup(uchar data[8])
{
  usbRequest_t *rq = (usbRequest_t*)data;

  return 0;  /* Return nothing to host for now */
}

//////////////////////////////////////////////////////////////
void sendKey(uint8_t keycode, uint8_t modifiers)
{
  reportKeyboard.key_code = keycode;
  reportKeyboard.modifiers = modifiers;
  usbSetInterrupt((uchar*)&reportKeyboard, sizeof(reportKeyboard));
}

//////////////////////////////////////////////////////////////
PT_THREAD(up_task(struct pt *pt))
{
  static uint32_t ts = 0;
  static uint8_t sequence = KEY_W;
  PT_BEGIN(pt);
  if(digitalRead(PIN_PC0)==LOW){
    sendKey(sequence,0);
    PT_DELAY(pt,10,ts);
    sendKey(KEY_NONE, 0);
  }
  PT_END(pt);
}
PT_THREAD(down_task(struct pt *pt))
{
  static uint32_t ts = 0;
  static uint8_t sequence = KEY_S;
  PT_BEGIN(pt);
  if(digitalRead(PIN_PC1)==LOW){
    sendKey(sequence,0);
    PT_DELAY(pt,10,ts);
    sendKey(KEY_NONE, 0);
  }
  PT_END(pt);
}
PT_THREAD(left_task(struct pt *pt))
{
  static uint32_t ts = 0;
  static uint8_t sequence = KEY_A;
  PT_BEGIN(pt);
  if(digitalRead(PIN_PC2)==LOW){
    sendKey(sequence,0);
    PT_DELAY(pt,10,ts);
    sendKey(KEY_NONE, 0);
  }
  PT_END(pt);
}
PT_THREAD(right_task(struct pt *pt))
{
  static uint32_t ts = 0;
  static uint8_t sequence = KEY_D;
  PT_BEGIN(pt);
  if(digitalRead(PIN_PC3)==LOW){
    sendKey(sequence,0);
    PT_DELAY(pt,10,ts);
    sendKey(KEY_NONE, 0);
  }
  PT_END(pt);
}
PT_THREAD(space_task(struct pt *pt))
{
  static uint32_t ts = 0;
  static uint8_t sequence = KEY_SPACE;
  PT_BEGIN(pt);
  if(digitalRead(PIN_PC4)==LOW){
    sendKey(sequence,0);
    PT_DELAY(pt,10,ts);
    sendKey(KEY_NONE, 0);
  }
  PT_END(pt);
}
//////////////////////////////////////////////////////////////
void setup()
{
  // Initialize USB subsystem
  usbInit();
  usbDeviceDisconnect();
  delay(300);
  usbDeviceConnect();
  
  // Initialize hardware pins
  pinMode(PIN_PC0, INPUT_PULLUP);
  pinMode(PIN_PC1, INPUT_PULLUP);
  pinMode(PIN_PC2, INPUT_PULLUP);
  pinMode(PIN_PC3, INPUT_PULLUP);
  pinMode(PIN_PC4, INPUT_PULLUP);

  // Initialize USB reports
  reportKeyboard.report_id = REPORT_ID_KEYBOARD;
  reportKeyboard.modifiers = 0;
  reportKeyboard.key_code = KEY_NONE;

  // Initialize tasks
  PT_INIT(&up_pt);
  PT_INIT(&down_pt);
  PT_INIT(&left_pt);
  PT_INIT(&right_pt);
  PT_INIT(&space_pt);
}

//////////////////////////////////////////////////////////////
void loop()
{
  usbPoll();
  up_task(&up_pt);
  down_task(&down_pt);
  left_task(&left_pt);
  right_task(&right_pt);
  space_task(&space_pt);
}
